﻿using Eingabeverwaltung;
using Bewertung;
using Ausgabeverwaltung;
using System;
using System.IO;
using System.Linq;

class ProgramForYapex
{
    private static SimpleEingabeverwalter eingabeverwalter;
    private static Ausgabeverwalter ausgabeverwalter;
    private static IBewerter Bewerter;
    private static bool SyntaxCheckOnly;
    private static string ConfigPath;

    static void Main(string[] args)
    {
        InitProgram(args);
        eingabeverwalter.verwalteEingabe();
        if (eingabeverwalter.IsProofParsed().HasValue && !eingabeverwalter.IsProofParsed().Value)
            Environment.Exit(1); // Bei Syntaxfehler im Beweis
        if (!eingabeverwalter.IsParsed().HasValue) throw new Exception("It was not tried parse the input by some unknown reason."); // this should not happen
        if (!eingabeverwalter.IsParsed().Value || SyntaxCheckOnly)
            Environment.Exit(0); // Bei Syntaxfehler in ADT / Task oder bei SyntaxCheckOnly
        else
        {
            ueberpruefe();
            bewerte();            
            ausgabe();
            bool isZeroPoints = true;
            foreach(IAssessmentResult ar in Bewerter.GetBewertung())
                isZeroPoints &= ar.Assessment() == 0.0;
            if (isZeroPoints) Environment.Exit(1);
            else Environment.Exit(0);
        }
    }

    private static void InitProgram(string[] args)
    {
        SyntaxCheckOnly = false;
        ConfigPath = "";
        string adtPath = "";
        string taskPath = "";
        string proofPath = "";
        foreach (string arg in args)
        {
            string fileEnding = arg.Split('.').Last();
            switch (fileEnding)
            {
                case "adt":
                    if (adtPath == "") adtPath = arg;
                    else throw new Exception("Found more than one adt in args!");
                    break;
                case "task":
                    if (taskPath == "") taskPath = arg;
                    else throw new Exception("Found more than one task in args!");
                    break;
                case "proof":
                    if (proofPath == "") proofPath = arg;
                    else throw new Exception("Found more than one proof in args!");
                    break;
                case "config":
                    if (ConfigPath == "") ConfigPath = arg;
                    else throw new Exception("Found more than one config in args!");
                    break;
                default:
                    if (arg == "-checkSyntaxOnly") SyntaxCheckOnly = true;
                    else throw new Exception("Found unexpected argument: " + arg + " !");
                    break;
            }
        }
        if (adtPath == "") throw new Exception("Missing path to ADT in the arguments! ADT must have fileending \".adt\".");
        if (taskPath == "") throw new Exception("Missing path to Task in the arguments! Task must have fileending \".task\".");
        if (proofPath == "") throw new Exception("Missing path to Proof in the arguments! Proof must have fileending \".proof\".");
        eingabeverwalter = new SimpleEingabeverwalter(adtPath,taskPath,proofPath);
    }

    private static void ueberpruefe() => Ueberpruefung.Ueberpruefer.ueberpruefe(eingabeverwalter.GetContainer());
    private static void bewerte()
    {
        if (ConfigPath == "") Bewerter = new Bewerter(eingabeverwalter.GetContainer());
        else Bewerter = new Bewerter(eingabeverwalter.GetContainer(), File.ReadAllText(ConfigPath));
        Bewerter.Bewerte();
    }
    private static void ausgabe()
    {
        ausgabeverwalter = new Ausgabeverwalter("string");
        ausgabeverwalter.Ausgabe(eingabeverwalter.GetContainer(), Bewerter.GetBewertung(), Bewerter.GetRound());
        Console.WriteLine(ausgabeverwalter.GetOutPut());
    }
}
